/*---------------------------------------------------------
 * Copyright (C) Microsoft Corporation. All rights reserved.
 *--------------------------------------------------------*/
'use strict';
var is = require('./is');
var messages_1 = require('./messages');
exports.ErrorCodes = messages_1.ErrorCodes;
var messageReader_1 = require('./messageReader');
var messageWriter_1 = require('./messageWriter');
function createMessageConnection(inputStream, outputStream, logger, client) {
    if (client === void 0) { client = false; }
    var protocolWriter = new messageWriter_1.MessageWriter(outputStream);
    var sequenceNumber = 0;
    var version = '2.0';
    var requestHandlers = Object.create(null);
    var responseHandlers = Object.create(null);
    var eventHandlers = Object.create(null);
    var connection = {
        sendNotification: function (type, params) {
            var notificatioMessage = {
                jsonrpc: version,
                method: type.method,
                params: params
            };
            protocolWriter.write(notificatioMessage);
        },
        onRequest: function (type, handler) {
            requestHandlers[type.method] = handler;
        },
        onNotification: function (type, handler) {
            eventHandlers[type.method] = handler;
        },
        dispose: function () {
        }
    };
    if (client) {
        connection.sendRequest = function (type, params) {
            return new Promise(function (resolve, reject) {
                var id = sequenceNumber++;
                var requestMessage = {
                    jsonrpc: version,
                    id: id,
                    method: type.method,
                    params: params
                };
                responseHandlers[String(id)] = { resolve: resolve, reject: reject };
                protocolWriter.write(requestMessage);
            });
        };
    }
    inputStream.on('end', function () { return outputStream.end(); });
    inputStream.on('close', function () { return outputStream.end(); });
    function handleRequest(requestMessage) {
        function reply(response) {
            var message = {
                jsonrpc: version,
                id: requestMessage.id
            };
            if (is.defined(response.error)) {
                message.error = response.error;
            }
            else if (is.defined(response.result)) {
                message.result = response.result;
            }
            else {
                message.error = {
                    code: messages_1.ErrorCodes.InvalidRequest,
                    message: "The request " + requestMessage.method + " did neither return a result nor a error."
                };
            }
            protocolWriter.write(message);
        }
        var requestHandler = requestHandlers[requestMessage.method];
        if (requestHandler) {
            try {
                var handlerResult = requestHandler(requestMessage.params);
                var promise = handlerResult;
                if (!promise) {
                    reply({ result: {} });
                }
                else if (promise.then) {
                    promise.then(function (response) {
                        reply(response);
                    }, function (error) {
                        if (messages_1.isFailedResponse(error)) {
                            reply(error);
                        }
                        else if (error && is.string(error.message)) {
                            reply({ error: { code: messages_1.ErrorCodes.InternalError, message: "Request " + requestMessage.method + " failed with message: " + error.message } });
                        }
                        else {
                            reply({ error: { code: messages_1.ErrorCodes.InternalError, message: "Request " + requestMessage.method + " failed unexpectedly without providing any details." } });
                        }
                    });
                }
                else {
                    reply(handlerResult);
                }
            }
            catch (error) {
                if (error && is.string(error.message)) {
                    reply({ error: { code: messages_1.ErrorCodes.InternalError, message: "Request " + requestMessage.method + " failed with message: " + error.message } });
                }
                else {
                    reply({ error: { code: messages_1.ErrorCodes.InternalError, message: "Request " + requestMessage.method + " failed unexpectedly without providing any details." } });
                }
            }
        }
        else {
            reply({ error: { code: messages_1.ErrorCodes.MethodNotFound, message: "Unhandled method " + requestMessage.method } });
        }
    }
    function handleResponse(responseMessage) {
        var key = String(responseMessage.id);
        var responseHandler = responseHandlers[key];
        if (responseHandler) {
            responseHandler.resolve(responseMessage);
            delete responseHandlers[String(responseMessage.id)];
        }
    }
    function handleNotification(message) {
        var eventHandler = eventHandlers[message.method];
        if (eventHandler) {
            try {
                eventHandler(message.params);
            }
            catch (error) {
                if (error.message) {
                    logger.error("Notification handler '" + message.method + "' failed with message: " + error.message);
                }
                else {
                    logger.error("Notification handler '" + message.method + "' failed unexpectedly.");
                }
            }
        }
    }
    var callback;
    if (client) {
        callback = function (message) {
            if (messages_1.isReponseMessage(message)) {
                handleResponse(message);
            }
            else if (messages_1.isNotificationMessage(message)) {
                handleNotification(message);
            }
        };
    }
    else {
        callback = function (message) {
            if (messages_1.isRequestMessage(message)) {
                handleRequest(message);
            }
            else if (messages_1.isNotificationMessage(message)) {
                handleNotification(message);
            }
        };
    }
    new messageReader_1.MessageReader(inputStream, callback);
    return connection;
}
function createServerMessageConnection(inputStream, outputStream, logger) {
    return createMessageConnection(inputStream, outputStream, logger);
}
exports.createServerMessageConnection = createServerMessageConnection;
function createClientMessageConnection(inputStream, outputStream, logger) {
    return createMessageConnection(inputStream, outputStream, logger, true);
}
exports.createClientMessageConnection = createClientMessageConnection;
//# sourceMappingURL=messageConnection.js.map