/**
 * Message kinds.
 */
export declare namespace Message {
    /**
     * A request message usually send from the client to the server
     */
    var Request: string;
    /**
     * A message response usually send from the server to the client
     */
    var Response: string;
    /**
     * A event message usually send asynchronously from the server to the client
     */
    var Event: string;
}
/**
 * A language server message
 */
export interface Message {
    jsonrpc: string;
}
/**
 * A Json RPC request message
 */
export interface Request {
    /**
     * The method's params
     */
    params?: any;
}
/**
 * Request message
 */
export interface RequestMessage extends Message, Request {
    /**
     * The request id;
     */
    id: number | string;
    /**
     * The method to be invoked
     */
    method: string;
}
/**
 * A interface to type the request parameter / response pair
 */
export interface RequestType<P, R extends Response> {
    method: string;
}
/**
 * Predefined error codes.
 */
export declare namespace ErrorCodes {
    const ParseError: number;
    const InvalidRequest: number;
    const MethodNotFound: number;
    const InvalidParams: number;
    const InternalError: number;
    const serverErrorStart: number;
    const serverErrorEnd: number;
}
/**
 * A error object return in a response in case a request
 * has failed.
 */
export interface ErrorInfo {
    /**
     * A number indicating the error type that occured
     */
    code: number;
    /**
     * A string providing a short decription of the error.
     */
    message: string;
    /**
     * A Primitive or Structured value that contains additional
     * information about the error. Can be omitted;
     */
    data?: any;
}
export interface Response {
    /**
     * The result of a request. This can be omitted in
     * the case of an error
     */
    result?: any;
    /**
     * The error object in case a request fails.
     */
    error?: ErrorInfo;
}
/**
 * A response message.
 */
export interface ResponseMessage extends Message, Response {
    /**
     * The request id;
     */
    id: number | string;
}
export interface Notification {
    /**
     * The notification's params
     */
    params?: any;
}
/**
 * Notification Message
 */
export interface NotificationMessage extends Message, Notification {
    /**
     * The method to be invoked
     */
    method: string;
}
export interface NotificationType<P> {
    method: string;
}
/**
 * Tests if the given message is a request message
 */
export declare function isRequestMessage(message: Message): message is RequestMessage;
/**
 * Tests if the given message is a notification message
 */
export declare function isNotificationMessage(message: Message): message is NotificationMessage;
/**
 * Tests if the given message is a response message
 */
export declare function isReponseMessage(message: Message): message is ResponseMessage;
export declare function isResponse(value: any): value is Response;
export declare function isFailedResponse(value: any): value is Response;
export declare function isSuccessfulResponse(value: any): value is Response;
